@@ -1,11 +1,11 @@
 ;;; elpa-mirror.el --- Create local package repository from installed packages
 
-;; Copyright (C) 2014-2020 Chen Bin
+;; Copyright (C) 2014-2024 Chen Bin
 
 ;; Author: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: http://github.com/redguardtoo/elpa-mirror
-;; Package-Requires: ((emacs "25.1"))
-;; Version: 2.2.2
+;; Package-Requires: ((emacs "27.1"))
+;; Version: 2.3.0
 ;; Keywords: tools
 ;;
 ;; This file is not part of GNU Emacs.
@@ -91,6 +91,8 @@
 (defcustom elpamr-tar-command-exclude-patterns
   '("*.elc"
     "*~"
+    "*/.gitignore"
+    "*/.git"
     "*autoloads.el"
     "*.so"
     "*.dylib"
@@ -292,7 +294,7 @@ command compatible with BSD tar instead of GNU tar."
 (defun elpamr-version ()
   "Current version."
   (interactive)
-  (message "2.2.2"))
+  (message "2.3.0"))
 
 (defun elpamr--win-executable-find (exe)
   "Find EXE on windows."
@@ -406,9 +408,8 @@ will be deleted and recreated."
 
         (dolist (package final-pkg-list)
           (let* ((pkg-dir (package-desc-dir (car (cdr package))))
-                 (name (package-desc-name (car (cdr package))))
                  (version-str (package-version-join (package-desc-version (car (cdr package)))))
-                 (name-fixed (format "%s-%s" name version-str))
+                 (pkg-dir-name (format "%s-%s" (package-desc-name (car (cdr package))) version-str))
                  (package-parent-dir-temp-p (not (string-match (concat "^" (file-truename package-user-dir))
                                                                pkg-dir)))
                  (package-parent-dir (cond
@@ -418,12 +419,14 @@ will be deleted and recreated."
                                        package-user-dir))))
 
             (when package-parent-dir-temp-p
-              (copy-directory pkg-dir (elpamr--fullpath tmp-dir name-fixed)))
+              (copy-directory pkg-dir (elpamr--fullpath tmp-dir pkg-dir-name)))
 
             (elpamr--run-tar package-parent-dir
-                             (file-relative-name (concat (elpamr--fullpath output-directory name-fixed) ".tar")
+                             (file-relative-name (concat (elpamr--fullpath output-directory pkg-dir-name) ".tar")
                                                  package-parent-dir)
-                             name-fixed
+                             ;; Since emacs 29, package might install from git repository
+                             ;; which doesn't create a directory containing version number
+                             (if package-parent-dir-temp-p pkg-dir-name (file-name-nondirectory pkg-dir))
                              is-bsd-tar)
             (message "Creating *.tar... %2d%% (%s)"
                      (/ (* cnt 100) (length final-pkg-list))
